LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;
USE ieee.std_logic_arith.all;

ENTITY FIFO IS
  Generic (Data_width: integer := 36;
           Fifo_size:  integer := 128);
  PORT( 
    clk              : IN     std_logic;
    flush              : IN     std_logic;
    Data_in          : IN     std_logic_vector (Data_width-1 DOWNTO 0); --Data input
    Data_out         : OUT    std_logic_vector (Data_width-1 DOWNTO 0)  --Data output
    );
END FIFO ;

ARCHITECTURE rtl OF FIFO IS
  -- log2_ceil based on a usenet post by Jonathan Bromley:
  -- http://groups.google.se/group/comp.lang.vhdl/browse_thread/thread/e63cd0f75b2535a1/5f8375970991806b
  function log2_ceil(N: natural) return positive is
  begin
    if N < 2 then
      return 1;
    else
      return 1 + log2_ceil(N/2);
    end if;
  end;

  type memory_type is array (0 to Fifo_size-1) of std_logic_vector(Data_width-1 DOWNTO 0);
  signal temp_mem : memory_type;

  constant REGLIMIT : integer := 10;     -- Adjust this to change the threshold
                                         -- for shiftreg implementation vs
                                         -- memory implementation.
begin

  assert REGLIMIT >= 2 report "Sanity check failed: REGLIMIT TOO SMALL" severity failure;

  -- Not a FIFO, but it may be useful to handle this case anyway
  NULLFIFO: if Fifo_size = 0 generate
    Data_out <= Data_in;
  end generate NULLFIFO;

  -- The FIFO is relatively small, implement using a shift register
  SMALLFIFO: if (Fifo_size <= REGLIMIT) and (Fifo_size > 0) generate 
    begin

      process(clk)
      begin
        if rising_edge(clk) then
          temp_mem(0) <= Data_in;
            if flush = '1' then
              temp_mem(0) <= (others => '0');
            end if;
        end if;
      end process;
          
      SHIFTREG: for i in 1 to Fifo_size-1 generate
      begin
        process(clk)
        begin
          if rising_edge(clk) then
            temp_mem(i) <= temp_mem(i-1);
            if flush = '1' then
              temp_mem(i) <= (others => '0');
            end if;
          end if;
        end process;
        
      end generate SHIFTREG;

    Data_out <= temp_mem(Fifo_size-1);
  end generate SMALLFIFO;

  -- The FIFO is relatively large, implement using memory
  LARGEFIFO: if Fifo_size > REGLIMIT generate
  signal wr_pointer  :std_logic_vector(log2_ceil(Fifo_size-1)-1 downto 0);
  signal re_pointer  :std_logic_vector(log2_ceil(Fifo_size-1)-1 downto 0);
  signal fifo_rden   : std_logic;  
  signal Q           :std_logic;
  begin

    Startwrite : process(clk)
    begin
      if rising_edge(clk) then
        Q <= '1';
        if (flush = '1') then
          Q <= '0';
        end if;
      end if;
    end process;
    
    Write:process(clk)
    begin
      if rising_edge(clk) then
        temp_mem(conv_integer(wr_pointer)) <= Data_in;
        if wr_pointer = Fifo_size - 2 then
          fifo_rden <= '1';
        end if;

        if Q = '1' then
          if wr_pointer = Fifo_size - 1 then
            wr_pointer <= (others => '0');
          else
            wr_pointer <= wr_pointer+1;
          end if;
        end if;

        if (flush = '1') then
          wr_pointer <= (others => '0');
          fifo_rden  <= '0';
        end if;
      end if;
    end process;

    Read : process(clk)
    begin
      if rising_edge(clk) then

        Data_out <= temp_mem(conv_integer(re_pointer));

        if fifo_rden = '1' then
          if re_pointer = Fifo_size - 1 then
            re_pointer <= (others => '0');
          else
            re_pointer <= re_pointer +1;
          end if;
        end if;
        if (flush = '1') then
          re_pointer <= (others => '0');
        end if;
      end if;
    end process;			 
    
  end generate LARGEFIFO;
end;
