
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_signed.all;
use ieee.numeric_std.all;
use ieee.std_logic_arith.all;
use ieee.math_real.all;

entity fft_processor_tb is
  generic(
    DATA_WIDTH : integer := 36
    );
end entity;
architecture test of fft_processor_tb is
--Signals interfacing the DUT and Behavioral Model
  signal clk    : std_logic := '0';
  signal rst    : std_logic := '1';
  signal Input  : std_logic_vector (DATA_WIDTH-1 downto 0);
  signal Output : std_logic_vector (DATA_WIDTH-1 downto 0);

  type   data_vector is array (0 to 255) of std_logic_vector(35 downto 0);
  signal data_input      : data_vector;
  signal expected_output : data_vector;

  signal tb_running : boolean := true;

  signal cyclecounter : integer := 0;
  
begin

  clk_gen : process
  begin
    while tb_running loop
      clk <= '0';
      wait for 25 ns;
      clk <= '1';
      wait for 25 ns;
    end loop;
    wait;
  end process;

  rst_gen : process
  begin
    rst <= '1';
    wait for 500 ns;

    wait until rising_edge(clk);        -- Make sure reset is released
                                        -- synchronously
    wait for 2 ns;
    rst <= '0';
    report "Reset released" severity note;
    wait;
  end process;


  -- Send the stimuli to the device under verification
  stimuli : process
  begin
    Input <= (others => '0');
    wait until rst = '0';
    wait until rising_edge(clk);

    report "Sending input data" severity note;
    for i in data_input'range loop
      wait for 2 ns;
      Input <= data_input(i);
      wait until rising_edge(clk);
    end loop;

    report "Done sending input data" severity note;
    wait;
  end process;

  sanity_checker : process
  begin
    for cyclecounter in 0 to 1000 loop
      wait until rising_edge(clk);
    end loop;  -- cyclecounter in range 0 to 1000

    report "*** FFT DID NOT PERFORM THE CORRECT FFT ! (Cyclecounter reached 1000) ***" severity failure;
    wait;
  end process;


  -- This comparison process will try to find the correct output value and
  -- start a comparison with the correct output values from the FFT.
  -- It is somewhat messy since it is capable of handling any latency up to
  -- 1000 cycles (at that point the sanity checker above will trigger and abort
  -- the test bench.
  comparison : process
    variable current_index : integer;
    variable finished      : boolean := false;
  begin
    wait until rst = '0';
    wait until rising_edge(clk);

    while not finished loop
      current_index := 0;

      report "Trying to find expected output values" severity note;
      while Output /= expected_output(0) loop
        wait until rising_edge(clk);
      end loop;
      report "Found expected_output(0) (cyclecounter is " &
        integer'image(cyclecounter) & ")" severity note;

      while (current_index < 256) and (Output = expected_output(current_index)) loop
        wait until rising_edge(clk);
        current_index := current_index + 1;
      end loop;

      if current_index = 256 then
        report "All values were correct" severity note;
        finished := true;
      else
        report "Unexpected value after " & integer'image(current_index) &
          " FFT output values, starting to scan again" severity note;

        if Output /= expected_output(0) then
          wait until rising_edge(clk);
        end if;
        
      end if;
    end loop;

    -- Stop clock (which will stop the sanity checking process above as well)
    tb_running <= false;                

    report "             ******* Testbench finished CORRECTLY *******" severity note;
    wait;

  end process;




  --Data Vector Generated from matlab script
  data_input(0)   <= "000000000111101100000000000101001000";
  data_input(1)   <= "000000001111010111000000001010001111";
  data_input(2)   <= "000000010111000011000000001111010111";
  data_input(3)   <= "000000011110101110000000010100011111";
  data_input(4)   <= "000000100110011010000000011001100110";
  data_input(5)   <= "000000101110000101000000011110101110";
  data_input(6)   <= "000000110101110001000000100011110110";
  data_input(7)   <= "000000111101011100000000101000111101";
  data_input(8)   <= "000001000101001000000000101110000101";
  data_input(9)   <= "000001001100110011000000110011001101";
  data_input(10)  <= "000001010100011111000000111000010100";
  data_input(11)  <= "000001011100001010000000111101011100";
  data_input(12)  <= "000001100011110110000001000010100100";
  data_input(13)  <= "000001101011100001000001000111101100";
  data_input(14)  <= "000001110011001101000001001100110011";
  data_input(15)  <= "000001111010111000000001010001111011";
  data_input(16)  <= "000010000010100100000001010111000011";
  data_input(17)  <= "000010001010001111000001011100001010";
  data_input(18)  <= "000010010001111011000001100001010010";
  data_input(19)  <= "000010011001100110000001100110011010";
  data_input(20)  <= "000010100001010010000001101011100001";
  data_input(21)  <= "000010101000111101000001110000101001";
  data_input(22)  <= "000010110000101001000001110101110001";
  data_input(23)  <= "000010111000010100000001111010111000";
  data_input(24)  <= "000011000000000000000010000000000000";
  data_input(25)  <= "000011000111101100000010000101001000";
  data_input(26)  <= "000011001111010111000010001010001111";
  data_input(27)  <= "000011010111000011000010001111010111";
  data_input(28)  <= "000011011110101110000010010100011111";
  data_input(29)  <= "000011100110011010000010011001100110";
  data_input(30)  <= "000011101110000101000010011110101110";
  data_input(31)  <= "000011110101110001000010100011110110";
  data_input(32)  <= "000001001100000010000000111011001001";
  data_input(33)  <= "000001001110010110000000111100111011";
  data_input(34)  <= "000001010000101001000000111110101110";
  data_input(35)  <= "000001010010111100000001000000100001";
  data_input(36)  <= "000001010101010000000001000010010011";
  data_input(37)  <= "000001010111100011000001000100000110";
  data_input(38)  <= "000001011001110111000001000101111001";
  data_input(39)  <= "000001011100001010000001000111101100";
  data_input(40)  <= "000001011110011110000001001001011110";
  data_input(41)  <= "000001100000110001000001001011010001";
  data_input(42)  <= "000001100011000101000001001101000100";
  data_input(43)  <= "000001100101011000000001001110110110";
  data_input(44)  <= "000001100111101100000001010000101001";
  data_input(45)  <= "000001101001111111000001010010011100";
  data_input(46)  <= "000001101100010010000001010100001110";
  data_input(47)  <= "000001101110100110000001010110000001";
  data_input(48)  <= "000001110000111001000001010111110100";
  data_input(49)  <= "000001110011001101000001011001100110";
  data_input(50)  <= "000001110101100000000001011011011001";
  data_input(51)  <= "000001110111110100000001011101001100";
  data_input(52)  <= "000001111010000111000001011110111110";
  data_input(53)  <= "000001111100011011000001100000110001";
  data_input(54)  <= "000001111110101110000001100010100100";
  data_input(55)  <= "000010000001000010000001100100010111";
  data_input(56)  <= "000010000011010101000001100110001001";
  data_input(57)  <= "000010000101101000000001100111111100";
  data_input(58)  <= "000010000111111100000001101001101111";
  data_input(59)  <= "000010001010001111000001101011100001";
  data_input(60)  <= "000010001100100011000001101101010100";
  data_input(61)  <= "000010001110110110000001101111000111";
  data_input(62)  <= "000010010001001010000001110000111001";
  data_input(63)  <= "000010010011011101000001110010101100";
  data_input(64)  <= "000001000010100100000000110001111011";
  data_input(65)  <= "000001000011100101000000110010101100";
  data_input(66)  <= "000001000100100111000000110011011101";
  data_input(67)  <= "000001000101101000000000110100001110";
  data_input(68)  <= "000001000110101010000000110100111111";
  data_input(69)  <= "000001000111101100000000110101110001";
  data_input(70)  <= "000001001000101101000000110110100010";
  data_input(71)  <= "000001001001101111000000110111010011";
  data_input(72)  <= "000001001010110000000000111000000100";
  data_input(73)  <= "000001001011110010000000111000110101";
  data_input(74)  <= "000001001100110011000000111001100110";
  data_input(75)  <= "000001001101110101000000111010011000";
  data_input(76)  <= "000001001110110110000000111011001001";
  data_input(77)  <= "000001001111111000000000111011111010";
  data_input(78)  <= "000001010000111001000000111100101011";
  data_input(79)  <= "000001010001111011000000111101011100";
  data_input(80)  <= "000001010010111100000000111110001101";
  data_input(81)  <= "000001010011111110000000111110111110";
  data_input(82)  <= "000001010100111111000000111111110000";
  data_input(83)  <= "000001010110000001000001000000100001";
  data_input(84)  <= "000001010111000011000001000001010010";
  data_input(85)  <= "000001011000000100000001000010000011";
  data_input(86)  <= "000001011001000110000001000010110100";
  data_input(87)  <= "000001011010000111000001000011100101";
  data_input(88)  <= "000001011011001001000001000100010111";
  data_input(89)  <= "000001011100001010000001000101001000";
  data_input(90)  <= "000001011101001100000001000101111001";
  data_input(91)  <= "000001011110001101000001000110101010";
  data_input(92)  <= "000001011111001111000001000111011011";
  data_input(93)  <= "000001100000010000000001001000001100";
  data_input(94)  <= "000001100001010010000001001000111101";
  data_input(95)  <= "000001100010010011000001001001101111";
  data_input(96)  <= "000010010100111111000001111100001010";
  data_input(97)  <= "000010010110100010000001111101011100";
  data_input(98)  <= "000010011000000100000001111110101110";
  data_input(99)  <= "000010011001100110000010000000000000";
  data_input(100) <= "000010011011001001000010000001010010";
  data_input(101) <= "000010011100101011000010000010100100";
  data_input(102) <= "000010011110001101000010000011110110";
  data_input(103) <= "000010011111110000000010000101001000";
  data_input(104) <= "000010100001010010000010000110011010";
  data_input(105) <= "000010100010110100000010000111101100";
  data_input(106) <= "000010100100010111000010001000111101";
  data_input(107) <= "000010100101111001000010001010001111";
  data_input(108) <= "000010100111011011000010001011100001";
  data_input(109) <= "000010101000111101000010001100110011";
  data_input(110) <= "000010101010100000000010001110000101";
  data_input(111) <= "000010101100000010000010001111010111";
  data_input(112) <= "000010101101100100000010010000101001";
  data_input(113) <= "000010101111000111000010010001111011";
  data_input(114) <= "000010110000101001000010010011001101";
  data_input(115) <= "000010110010001011000010010100011111";
  data_input(116) <= "000010110011101110000010010101110001";
  data_input(117) <= "000010110101010000000010010111000011";
  data_input(118) <= "000010110110110010000010011000010100";
  data_input(119) <= "000010111000010100000010011001100110";
  data_input(120) <= "000010111001110111000010011010111000";
  data_input(121) <= "000010111011011001000010011100001010";
  data_input(122) <= "000010111100111011000010011101011100";
  data_input(123) <= "000010111110011110000010011110101110";
  data_input(124) <= "000011000000000000000010100000000000";
  data_input(125) <= "000011000001100010000010100001010010";
  data_input(126) <= "000011000011000101000010100010100100";
  data_input(127) <= "000011000100100111000010100011110110";
  data_input(128) <= "000001000010000011000001100011000101";
  data_input(129) <= "000001000010100100000001100011110110";
  data_input(130) <= "000001000011000101000001100100100111";
  data_input(131) <= "000001000011100101000001100101011000";
  data_input(132) <= "000001000100000110000001100110001001";
  data_input(133) <= "000001000100100111000001100110111010";
  data_input(134) <= "000001000101001000000001100111101100";
  data_input(135) <= "000001000101101000000001101000011101";
  data_input(136) <= "000001000110001001000001101001001110";
  data_input(137) <= "000001000110101010000001101001111111";
  data_input(138) <= "000001000111001011000001101010110000";
  data_input(139) <= "000001000111101100000001101011100001";
  data_input(140) <= "000001001000001100000001101100010010";
  data_input(141) <= "000001001000101101000001101101000100";
  data_input(142) <= "000001001001001110000001101101110101";
  data_input(143) <= "000001001001101111000001101110100110";
  data_input(144) <= "000001001010001111000001101111010111";
  data_input(145) <= "000001001010110000000001110000001000";
  data_input(146) <= "000001001011010001000001110000111001";
  data_input(147) <= "000001001011110010000001110001101010";
  data_input(148) <= "000001001100010010000001110010011100";
  data_input(149) <= "000001001100110011000001110011001101";
  data_input(150) <= "000001001101010100000001110011111110";
  data_input(151) <= "000001001101110101000001110100101111";
  data_input(152) <= "000001001110010110000001110101100000";
  data_input(153) <= "000001001110110110000001110110010001";
  data_input(154) <= "000001001111010111000001110111000011";
  data_input(155) <= "000001001111111000000001110111110100";
  data_input(156) <= "000001010000011001000001111000100101";
  data_input(157) <= "000001010000111001000001111001010110";
  data_input(158) <= "000001010001011010000001111010000111";
  data_input(159) <= "000001010001111011000001111010111000";
  data_input(160) <= "000001010010011100000001111011101001";
  data_input(161) <= "000001010010111100000001111100011011";
  data_input(162) <= "000001010011011101000001111101001100";
  data_input(163) <= "000001010011111110000001111101111101";
  data_input(164) <= "000001010100011111000001111110101110";
  data_input(165) <= "000001010100111111000001111111011111";
  data_input(166) <= "000001010101100000000010000000010000";
  data_input(167) <= "000001010110000001000010000001000010";
  data_input(168) <= "000001010110100010000010000001110011";
  data_input(169) <= "000001010111000011000010000010100100";
  data_input(170) <= "000001010111100011000010000011010101";
  data_input(171) <= "000001011000000100000010000100000110";
  data_input(172) <= "000001011000100101000010000100110111";
  data_input(173) <= "000001011001000110000010000101101000";
  data_input(174) <= "000001011001100110000010000110011010";
  data_input(175) <= "000001011010000111000010000111001011";
  data_input(176) <= "000001011010101000000010000111111100";
  data_input(177) <= "000001011011001001000010001000101101";
  data_input(178) <= "000001011011101001000010001001011110";
  data_input(179) <= "000001011100001010000010001010001111";
  data_input(180) <= "000001011100101011000010001011000001";
  data_input(181) <= "000001011101001100000010001011110010";
  data_input(182) <= "000001011101101101000010001100100011";
  data_input(183) <= "000001011110001101000010001101010100";
  data_input(184) <= "000001011110101110000010001110000101";
  data_input(185) <= "000001011111001111000010001110110110";
  data_input(186) <= "000001011111110000000010001111100111";
  data_input(187) <= "000001100000010000000010010000011001";
  data_input(188) <= "000001100000110001000010010001001010";
  data_input(189) <= "000001100001010010000010010001111011";
  data_input(190) <= "000001100001110011000010010010101100";
  data_input(191) <= "000001100010010011000010010011011101";
  data_input(192) <= "000000110001011010000001100010110100";
  data_input(193) <= "000000110001101010000001100011010101";
  data_input(194) <= "000000110001111011000001100011110110";
  data_input(195) <= "000000110010001011000001100100010111";
  data_input(196) <= "000000110010011100000001100100110111";
  data_input(197) <= "000000110010101100000001100101011000";
  data_input(198) <= "000000110010111100000001100101111001";
  data_input(199) <= "000000110011001101000001100110011010";
  data_input(200) <= "000000110011011101000001100110111010";
  data_input(201) <= "000000110011101110000001100111011011";
  data_input(202) <= "000000110011111110000001100111111100";
  data_input(203) <= "000000110100001110000001101000011101";
  data_input(204) <= "000000110100011111000001101000111101";
  data_input(205) <= "000000110100101111000001101001011110";
  data_input(206) <= "000000110100111111000001101001111111";
  data_input(207) <= "000000110101010000000001101010100000";
  data_input(208) <= "000000110101100000000001101011000001";
  data_input(209) <= "000000110101110001000001101011100001";
  data_input(210) <= "000000110110000001000001101100000010";
  data_input(211) <= "000000110110010001000001101100100011";
  data_input(212) <= "000000110110100010000001101101000100";
  data_input(213) <= "000000110110110010000001101101100100";
  data_input(214) <= "000000110111000011000001101110000101";
  data_input(215) <= "000000110111010011000001101110100110";
  data_input(216) <= "000000110111100011000001101111000111";
  data_input(217) <= "000000110111110100000001101111100111";
  data_input(218) <= "000000111000000100000001110000001000";
  data_input(219) <= "000000111000010100000001110000101001";
  data_input(220) <= "000000111000100101000001110001001010";
  data_input(221) <= "000000111000110101000001110001101010";
  data_input(222) <= "000000111001000110000001110010001011";
  data_input(223) <= "000000111001010110000001110010101100";
  data_input(224) <= "000010010000000000000011001001100110";
  data_input(225) <= "000010010000101001000011001010100000";
  data_input(226) <= "000010010001010010000011001011011001";
  data_input(227) <= "000010010001111011000011001100010010";
  data_input(228) <= "000010010010100100000011001101001100";
  data_input(229) <= "000010010011001101000011001110000101";
  data_input(230) <= "000010010011110110000011001110111110";
  data_input(231) <= "000010010100011111000011001111111000";
  data_input(232) <= "000010010101001000000011010000110001";
  data_input(233) <= "000010010101110001000011010001101010";
  data_input(234) <= "000010010110011010000011010010100100";
  data_input(235) <= "000010010111000011000011010011011101";
  data_input(236) <= "000010010111101100000011010100010111";
  data_input(237) <= "000010011000010100000011010101010000";
  data_input(238) <= "000010011000111101000011010110001001";
  data_input(239) <= "000010011001100110000011010111000011";
  data_input(240) <= "000010011010001111000011010111111100";
  data_input(241) <= "000010011010111000000011011000110101";
  data_input(242) <= "000010011011100001000011011001101111";
  data_input(243) <= "000010011100001010000011011010101000";
  data_input(244) <= "000010011100110011000011011011100001";
  data_input(245) <= "000010011101011100000011011100011011";
  data_input(246) <= "000010011110000101000011011101010100";
  data_input(247) <= "000010011110101110000011011110001101";
  data_input(248) <= "000010011111010111000011011111000111";
  data_input(249) <= "000010100000000000000011100000000000";
  data_input(250) <= "000010100000101001000011100000111001";
  data_input(251) <= "000010100001010010000011100001110011";
  data_input(252) <= "000010100001111011000011100010101100";
  data_input(253) <= "000010100010100100000011100011100101";
  data_input(254) <= "000010100011001101000011100100011111";
  data_input(255) <= "000010100011110110000011100101011000";

  -- Expected output, verified against matlab
  -- Should not be changed during the lab
  -- (At least not without discussing it with the examiners)
  expected_output(0)   <= "000000000001101011000000000001110101";
  expected_output(1)   <= "111111111111111111111111111111111110";
  expected_output(2)   <= "111111111111111110111111111111111111";
  expected_output(3)   <= "111111111111111111111111111111111110";
  expected_output(4)   <= "111111111111111100000000000000000000";
  expected_output(5)   <= "111111111111111111111111111111111110";
  expected_output(6)   <= "111111111111111111111111111111111111";
  expected_output(7)   <= "000000000000000001111111111111111100";
  expected_output(8)   <= "111111111111111010000000000000000011";
  expected_output(9)   <= "111111111111111111111111111111111110";
  expected_output(10)  <= "111111111111111110111111111111111111";
  expected_output(11)  <= "000000000000000000111111111111111101";
  expected_output(12)  <= "111111111111111101000000000000000000";
  expected_output(13)  <= "111111111111111111111111111111111110";
  expected_output(14)  <= "111111111111111111111111111111111111";
  expected_output(15)  <= "000000000000000011111111111111111010";
  expected_output(16)  <= "111111111111110111000000000000000111";
  expected_output(17)  <= "111111111111111111111111111111111111";
  expected_output(18)  <= "111111111111111110111111111111111111";
  expected_output(19)  <= "000000000000000000111111111111111110";
  expected_output(20)  <= "111111111111111100000000000000000000";
  expected_output(21)  <= "111111111111111111111111111111111110";
  expected_output(22)  <= "111111111111111110111111111111111111";
  expected_output(23)  <= "000000000000000001111111111111111100";
  expected_output(24)  <= "111111111111111011000000000000000001";
  expected_output(25)  <= "111111111111111111111111111111111110";
  expected_output(26)  <= "111111111111111110111111111111111111";
  expected_output(27)  <= "000000000000000000111111111111111101";
  expected_output(28)  <= "111111111111111101111111111111111111";
  expected_output(29)  <= "111111111111111111111111111111111110";
  expected_output(30)  <= "111111111111111110111111111111111110";
  expected_output(31)  <= "000000000000000110111111111111110110";
  expected_output(32)  <= "111111111111101001000000000000001011";
  expected_output(33)  <= "111111111111111111111111111111111110";
  expected_output(34)  <= "111111111111111110111111111111111111";
  expected_output(35)  <= "000000000000000000111111111111111110";
  expected_output(36)  <= "111111111111111100000000000000000000";
  expected_output(37)  <= "111111111111111111111111111111111110";
  expected_output(38)  <= "111111111111111110111111111111111111";
  expected_output(39)  <= "000000000000000001111111111111111100";
  expected_output(40)  <= "111111111111111010000000000000000001";
  expected_output(41)  <= "111111111111111111111111111111111110";
  expected_output(42)  <= "111111111111111110111111111111111111";
  expected_output(43)  <= "000000000000000000111111111111111101";
  expected_output(44)  <= "111111111111111101111111111111111111";
  expected_output(45)  <= "000000000000000000111111111111111110";
  expected_output(46)  <= "111111111111111110111111111111111111";
  expected_output(47)  <= "000000000000000100111111111111111001";
  expected_output(48)  <= "111111111111111000000000000000000011";
  expected_output(49)  <= "111111111111111111111111111111111110";
  expected_output(50)  <= "111111111111111110111111111111111111";
  expected_output(51)  <= "000000000000000000111111111111111110";
  expected_output(52)  <= "111111111111111101111111111111111111";
  expected_output(53)  <= "000000000000000000111111111111111110";
  expected_output(54)  <= "111111111111111110111111111111111111";
  expected_output(55)  <= "000000000000000010111111111111111011";
  expected_output(56)  <= "111111111111111100000000000000000001";
  expected_output(57)  <= "000000000000000000111111111111111110";
  expected_output(58)  <= "111111111111111111111111111111111111";
  expected_output(59)  <= "000000000000000001111111111111111101";
  expected_output(60)  <= "111111111111111101111111111111111111";
  expected_output(61)  <= "000000000000000000111111111111111110";
  expected_output(62)  <= "111111111111111111111111111111111111";
  expected_output(63)  <= "000000000000001101111111111111101101";
  expected_output(64)  <= "000000000000000011000000000000001100";
  expected_output(65)  <= "111111111111111111000000000000000000";
  expected_output(66)  <= "111111111111111111000000000000000000";
  expected_output(67)  <= "111111111111111111111111111111111111";
  expected_output(68)  <= "111111111111111111000000000000000001";
  expected_output(69)  <= "111111111111111111111111111111111111";
  expected_output(70)  <= "111111111111111111000000000000000000";
  expected_output(71)  <= "111111111111111111111111111111111101";
  expected_output(72)  <= "111111111111111111000000000000000011";
  expected_output(73)  <= "111111111111111111111111111111111111";
  expected_output(74)  <= "111111111111111111000000000000000000";
  expected_output(75)  <= "111111111111111111111111111111111111";
  expected_output(76)  <= "111111111111111111000000000000000000";
  expected_output(77)  <= "111111111111111111111111111111111111";
  expected_output(78)  <= "111111111111111111111111111111111111";
  expected_output(79)  <= "111111111111111110111111111111111011";
  expected_output(80)  <= "000000000000000000000000000000000101";
  expected_output(81)  <= "111111111111111111111111111111111111";
  expected_output(82)  <= "111111111111111111000000000000000000";
  expected_output(83)  <= "111111111111111110111111111111111111";
  expected_output(84)  <= "111111111111111111000000000000000001";
  expected_output(85)  <= "111111111111111111111111111111111111";
  expected_output(86)  <= "111111111111111111000000000000000000";
  expected_output(87)  <= "111111111111111110111111111111111101";
  expected_output(88)  <= "111111111111111111000000000000000010";
  expected_output(89)  <= "111111111111111111111111111111111111";
  expected_output(90)  <= "111111111111111111000000000000000000";
  expected_output(91)  <= "111111111111111110111111111111111110";
  expected_output(92)  <= "111111111111111111000000000000000000";
  expected_output(93)  <= "111111111111111111111111111111111111";
  expected_output(94)  <= "111111111111111111000000000000000000";
  expected_output(95)  <= "111111111111111011111111111111110011";
  expected_output(96)  <= "111111111111110000111111111111111100";
  expected_output(97)  <= "000000000000000000111111111111111110";
  expected_output(98)  <= "111111111111111111111111111111111110";
  expected_output(99)  <= "000000000000000001111111111111111111";
  expected_output(100) <= "111111111111111101111111111111111110";
  expected_output(101) <= "000000000000000000111111111111111110";
  expected_output(102) <= "111111111111111111111111111111111110";
  expected_output(103) <= "000000000000000010111111111111111111";
  expected_output(104) <= "111111111111111011111111111111111110";
  expected_output(105) <= "000000000000000000111111111111111110";
  expected_output(106) <= "111111111111111111111111111111111110";
  expected_output(107) <= "000000000000000001111111111111111111";
  expected_output(108) <= "111111111111111110111111111111111110";
  expected_output(109) <= "000000000000000001111111111111111110";
  expected_output(110) <= "111111111111111111111111111111111110";
  expected_output(111) <= "000000000000000111000000000000000000";
  expected_output(112) <= "111111111111111001111111111111111101";
  expected_output(113) <= "000000000000000000111111111111111110";
  expected_output(114) <= "111111111111111111111111111111111110";
  expected_output(115) <= "000000000000000001111111111111111111";
  expected_output(116) <= "111111111111111110111111111111111110";
  expected_output(117) <= "000000000000000001111111111111111110";
  expected_output(118) <= "111111111111111111111111111111111110";
  expected_output(119) <= "000000000000000100111111111111111111";
  expected_output(120) <= "111111111111111100111111111111111110";
  expected_output(121) <= "000000000000000000111111111111111110";
  expected_output(122) <= "111111111111111111111111111111111110";
  expected_output(123) <= "000000000000000010111111111111111111";
  expected_output(124) <= "111111111111111111111111111111111110";
  expected_output(125) <= "000000000000000001111111111111111110";
  expected_output(126) <= "000000000000000000111111111111111110";
  expected_output(127) <= "000000000000010100000000000000001000";
  expected_output(128) <= "111111111111110011111111111111111000";
  expected_output(129) <= "111111111111111111111111111111111111";
  expected_output(130) <= "111111111111111111111111111111111111";
  expected_output(131) <= "111111111111111111111111111111111111";
  expected_output(132) <= "111111111111111110000000000000000000";
  expected_output(133) <= "111111111111111111111111111111111111";
  expected_output(134) <= "111111111111111111000000000000000000";
  expected_output(135) <= "111111111111111111111111111111111110";
  expected_output(136) <= "111111111111111110000000000000000001";
  expected_output(137) <= "111111111111111111111111111111111111";
  expected_output(138) <= "111111111111111111111111111111111111";
  expected_output(139) <= "111111111111111111111111111111111111";
  expected_output(140) <= "111111111111111110000000000000000000";
  expected_output(141) <= "111111111111111111111111111111111111";
  expected_output(142) <= "111111111111111111111111111111111111";
  expected_output(143) <= "000000000000000000111111111111111101";
  expected_output(144) <= "111111111111111110000000000000000010";
  expected_output(145) <= "111111111111111111111111111111111111";
  expected_output(146) <= "111111111111111111111111111111111111";
  expected_output(147) <= "111111111111111111111111111111111111";
  expected_output(148) <= "111111111111111110000000000000000000";
  expected_output(149) <= "111111111111111111111111111111111111";
  expected_output(150) <= "111111111111111111111111111111111111";
  expected_output(151) <= "000000000000000000111111111111111110";
  expected_output(152) <= "111111111111111110000000000000000000";
  expected_output(153) <= "111111111111111111111111111111111111";
  expected_output(154) <= "111111111111111111111111111111111111";
  expected_output(155) <= "111111111111111111111111111111111111";
  expected_output(156) <= "111111111111111110111111111111111111";
  expected_output(157) <= "111111111111111111111111111111111111";
  expected_output(158) <= "111111111111111111111111111111111111";
  expected_output(159) <= "000000000000000000111111111111111001";
  expected_output(160) <= "111111111111110100111111111111111000";
  expected_output(161) <= "000000000000000000111111111111111111";
  expected_output(162) <= "111111111111111111111111111111111111";
  expected_output(163) <= "000000000000000000111111111111111111";
  expected_output(164) <= "111111111111111110111111111111111110";
  expected_output(165) <= "000000000000000000111111111111111111";
  expected_output(166) <= "111111111111111111111111111111111111";
  expected_output(167) <= "000000000000000001000000000000000000";
  expected_output(168) <= "111111111111111101111111111111111101";
  expected_output(169) <= "000000000000000000111111111111111111";
  expected_output(170) <= "000000000000000000111111111111111111";
  expected_output(171) <= "000000000000000001111111111111111111";
  expected_output(172) <= "111111111111111111111111111111111110";
  expected_output(173) <= "000000000000000000111111111111111111";
  expected_output(174) <= "000000000000000000111111111111111111";
  expected_output(175) <= "000000000000000011000000000000000001";
  expected_output(176) <= "111111111111111100111111111111111100";
  expected_output(177) <= "000000000000000000111111111111111111";
  expected_output(178) <= "000000000000000000111111111111111110";
  expected_output(179) <= "000000000000000001000000000000000000";
  expected_output(180) <= "111111111111111110111111111111111110";
  expected_output(181) <= "000000000000000000111111111111111111";
  expected_output(182) <= "000000000000000000111111111111111111";
  expected_output(183) <= "000000000000000010000000000000000000";
  expected_output(184) <= "111111111111111110111111111111111101";
  expected_output(185) <= "000000000000000000111111111111111111";
  expected_output(186) <= "111111111111111111111111111111111110";
  expected_output(187) <= "000000000000000001000000000000000000";
  expected_output(188) <= "111111111111111111111111111111111110";
  expected_output(189) <= "000000000000000001111111111111111111";
  expected_output(190) <= "000000000000000000111111111111111111";
  expected_output(191) <= "000000000000000010000000000000000100";
  expected_output(192) <= "111111111111111000111111111111111001";
  expected_output(193) <= "000000000000000000111111111111111111";
  expected_output(194) <= "000000000000000000111111111111111111";
  expected_output(195) <= "000000000000000000000000000000000000";
  expected_output(196) <= "111111111111111111111111111111111111";
  expected_output(197) <= "000000000000000000111111111111111111";
  expected_output(198) <= "111111111111111111111111111111111111";
  expected_output(199) <= "111111111111111111111111111111111111";
  expected_output(200) <= "111111111111111111111111111111111111";
  expected_output(201) <= "000000000000000000111111111111111111";
  expected_output(202) <= "000000000000000000111111111111111111";
  expected_output(203) <= "000000000000000000000000000000000000";
  expected_output(204) <= "111111111111111111111111111111111111";
  expected_output(205) <= "000000000000000000111111111111111111";
  expected_output(206) <= "111111111111111111111111111111111111";
  expected_output(207) <= "111111111111111111111111111111111111";
  expected_output(208) <= "111111111111111111111111111111111111";
  expected_output(209) <= "000000000000000000111111111111111111";
  expected_output(210) <= "000000000000000000111111111111111111";
  expected_output(211) <= "000000000000000000000000000000000000";
  expected_output(212) <= "111111111111111111111111111111111111";
  expected_output(213) <= "000000000000000000111111111111111111";
  expected_output(214) <= "111111111111111111111111111111111111";
  expected_output(215) <= "111111111111111111111111111111111111";
  expected_output(216) <= "111111111111111111111111111111111111";
  expected_output(217) <= "000000000000000000111111111111111111";
  expected_output(218) <= "000000000000000000111111111111111111";
  expected_output(219) <= "000000000000000000111111111111111111";
  expected_output(220) <= "111111111111111111000000000000000000";
  expected_output(221) <= "111111111111111111111111111111111111";
  expected_output(222) <= "111111111111111111111111111111111111";
  expected_output(223) <= "111111111111111100111111111111111101";
  expected_output(224) <= "111111111111110101111111111111111110";
  expected_output(225) <= "000000000000000000111111111111111111";
  expected_output(226) <= "111111111111111111111111111111111110";
  expected_output(227) <= "000000000000000001111111111111111111";
  expected_output(228) <= "111111111111111101111111111111111110";
  expected_output(229) <= "000000000000000000111111111111111111";
  expected_output(230) <= "111111111111111111111111111111111110";
  expected_output(231) <= "000000000000000010111111111111111111";
  expected_output(232) <= "111111111111111100111111111111111110";
  expected_output(233) <= "000000000000000000111111111111111111";
  expected_output(234) <= "111111111111111111111111111111111110";
  expected_output(235) <= "000000000000000001111111111111111111";
  expected_output(236) <= "111111111111111110111111111111111110";
  expected_output(237) <= "000000000000000000111111111111111111";
  expected_output(238) <= "111111111111111111111111111111111110";
  expected_output(239) <= "000000000000000110111111111111111111";
  expected_output(240) <= "111111111111111011111111111111111110";
  expected_output(241) <= "000000000000000000111111111111111111";
  expected_output(242) <= "111111111111111111111111111111111110";
  expected_output(243) <= "000000000000000001111111111111111111";
  expected_output(244) <= "111111111111111110111111111111111110";
  expected_output(245) <= "000000000000000000111111111111111111";
  expected_output(246) <= "111111111111111111111111111111111110";
  expected_output(247) <= "000000000000000011111111111111111111";
  expected_output(248) <= "111111111111111101111111111111111110";
  expected_output(249) <= "000000000000000000111111111111111111";
  expected_output(250) <= "111111111111111111111111111111111110";
  expected_output(251) <= "000000000000000010111111111111111111";
  expected_output(252) <= "111111111111111110111111111111111110";
  expected_output(253) <= "000000000000000000111111111111111110";
  expected_output(254) <= "111111111111111111111111111111111110";
  expected_output(255) <= "000000000000010000000000000000001110";


  uut : entity work.fft_processor
--    generic map(
--      DATA_WIDTH   => 36
--      )
    port map(
      clk    => clk,
      rst    => rst,
      Input  => Input,
      Output => Output
      );
end architecture;
